@@ -34,8 +34,14 @@ from c8ylp.websocket_client.ws_client import WebsocketClient
 
 PIDFILE = '/var/run/c8ylp/c8ylp'
 
+class ExitCommand(Exception):
+    pass
+
+def signal_handler(signal, frame):
+    raise ExitCommand()
 
 def start():
+    signal.signal(signal.SIGUSR1, signal_handler)
     home = expanduser('~')
     path = pathlib.Path(home + '/.c8ylp')
     loglevel = logging.INFO
@@ -68,7 +74,7 @@ def start():
                                    ["help", "hostname=", "device=", "extype=", "config=", "tenant=", "username=",
                                     "password=", "tfacode=", "port=", "kill", "tcpsize=", "tcptimeout=", "verbose",
                                     "scriptmode",
-                                    "ignore-ssl-validate"])
+                                    "ignore-ssl-validate", "use-pid", "reconnects="])
     except getopt.GetoptError as e:
         logging.error(e)
         help()
@@ -85,12 +91,15 @@ def start():
     user = os.environ.get('C8Y_USER')
     password = os.environ.get('C8Y_PASSWORD')
     tcp_size = int(os.environ.get('C8Y_TCPSIZE')) if os.environ.get('C8Y_TCPSIZE') is not None else 32768
-    tcp_timeout = int(os.environ.get('C8Y_TCPTIMEOUT')) if os.environ.get('C8Y_TCPTIMEOUT') is not None else 60
+    tcp_timeout = int(os.environ.get('C8Y_TCPTIMEOUT')) if os.environ.get('C8Y_TCPTIMEOUT') is not None else 0
     port = int(os.environ.get('C8Y_PORT')) if os.environ.get('C8Y_PORT') is not None else 2222
     token = os.environ.get('C8Y_TOKEN')
     tfacode = None
     script_mode = False
     ignore_ssl_validate = False
+    use_pid = False
+    kill_instances = False
+    reconnects = 5
     for option_key, option_value in opts:
         if option_key in ('-h', '--hostname'):
             host = option_value
@@ -111,7 +120,7 @@ def start():
         elif option_key in ['--port']:
             port = int(option_value)
         elif option_key in ['-k', '--kill']:
-            kill_existing_instances()
+            kill_instances = True
         elif option_key in ['--tcpsize']:
             tcp_size = int(option_value)
         elif option_key in ['--tcptimeout']:
@@ -122,27 +131,40 @@ def start():
             script_mode = True
         elif option_key in ['--ignore-ssl-validate']:
             ignore_ssl_validate = True
+        elif option_key in ['--use-pid']:
+            use_pid = True
+        elif option_key in ['--reconnects']:
+            reconnects = int(option_value)
         elif option_key in ['--help']:
             help()
     validate_parameter(host, device, extype, config_name,
                        tenant, user, password, token)
-    upsert_pid_file(device, host, config_name, user)
+    if use_pid:
+        upsert_pid_file(device, host, config_name, user)
+    if kill_instances:
+        if use_pid:
+            kill_existing_instances()
+        else:
+            logging.warn(f'WARNING: Killing existing instances is only support when "--use-pid" is used.')
     client = CumulocityClient(host, tenant, user, password, tfacode, ignore_ssl_validate)
+    tenant_id_valid = client.validate_tenant_id()
+    if tenant_id_valid is not None:
+        logging.warn(f'WARNING: Tenant ID {tenant} does not exist. Try using this Tenant ID {tenant_id_valid} next time!')
     session = None
     if token:
         client.validate_token()
     else:
         session = client.retrieve_token()
-
     mor = client.read_mo(device, extype)
     config_id = client.get_config_id(mor, config_name)
     device_id = client.get_device_id(mor)
+    
     is_authorized = client.validate_remote_access_role()
     if not is_authorized:
         logging.error(f'User {user} is not authorized to use Cloud Remote Access. Contact your Cumulocity Admin!')
         sys.exit(1)
     websocket_client = WebsocketClient(
-        host, tenant, user, password, config_id, device_id, session, token, ignore_ssl_validate)
+        host, tenant, user, password, config_id, device_id, session, token, ignore_ssl_validate, reconnects)
     wst = websocket_client.connect()
     tcp_server = TCPServer(port, websocket_client, tcp_size, tcp_timeout, wst, script_mode)
     # TCP is blocking...
@@ -152,7 +174,8 @@ def start():
     except Exception as ex:
         logging.error(f'Error on TCP-Server {ex}')
     finally:
-        clean_pid_file(None)
+        if use_pid:
+            clean_pid_file(None)
         tcp_server.stop()
         sys.exit(0)
 
@@ -221,9 +244,11 @@ def clean_pid_file(pid):
 def kill_existing_instances():
     if os.path.exists(PIDFILE):
         file = open(PIDFILE)
+        pid = int(os.getpid())
+        #logging.info(f'Current PID {pid}')
         for line in file:
             other_pid = get_pid_from_line(line)
-            if pid_is_active(other_pid):
+            if pid != other_pid and pid_is_active(other_pid):
                 logging.info(
                     f'Killing other running Process with PID {other_pid}')
                 os.kill(get_pid_from_line(line), 9)
@@ -289,10 +314,12 @@ def _help_message() -> str:
                ' --port                 OPTIONAL, the TCP Port which should be opened. Default: 2222\n'
                ' -k, --kill             OPTIONAL, kills all existing processes of c8ylp\n'
                ' --tcpsize              OPTIONAL, the TCP Package Size. Default: 32768\n'
-               ' --tcptimeout           OPTIONAL, Timeout in sec. for inactivity. Can be deactivited with "0". Default: 60 sec.\n'
+               ' --tcptimeout           OPTIONAL, Timeout in sec. for inactivity. Can be activited with values > 0. Default: Deactivated\n'
                ' -v, --verbose          OPTIONAL, Print Debug Information into the Logs and Console when set.\n'
                ' -s, --scriptmode       OPTIONAL, Stops the TCP Server after first connection. No automatical restart!\n'
                ' --ignore-ssl-validate  OPTIONAL, Ignore Validation for SSL Certificates while connecting to Websocket'
+               ' --use-pid              OPTIONAL, Will create a PID-File in /var/run/c8ylp to store all Processes currently running.\n'
+               ' --reconnects           OPTIONAL, The number of reconnects to the Cloud Remote Service. 0 for infinite reconnects. Default: 5'
                '\n')
 
 
